#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Plug-in:      Layer via Copy/Cut
# Version:      1.5
# Date:         04.06.2012
# Copyright:    Dmitry Dubyaga <dmitry.dubyaga@gmail.com>
# Tested with:  GIMP 2.8
# ----------------------------------

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from gimpfu import *

def show_message(m_id):
    messages = []
    messages.append("The layer or layer group is not selected.")
    messages.append("The selection is empty.")
    pdb.gimp_message(messages[m_id])
    return

def find_layers_in_group(group, elements):
    num_children, child_ids = pdb.gimp_item_get_children(group)
    for id in child_ids:
        child = gimp.Item.from_id(id)
        if pdb.gimp_item_is_group(child):
            find_layers_in_group(child, elements)
        else:
            elements.append(child)
    return elements

def python_fu_layer_via_copy(image, drawable):
    gimp.context_push()
    image.undo_group_start()

    pdb.gimp_message_get_handler(ERROR_CONSOLE)

    if not pdb.gimp_item_is_layer(drawable):
        show_message(0)
        image.undo_group_end()
        gimp.context_pop()
        return

    if pdb.gimp_selection_is_empty(image):
        show_message(1)
        image.undo_group_end()
        gimp.context_pop()
        return

    if pdb.gimp_item_is_group(drawable):

        parent = pdb.gimp_item_get_parent(drawable)
        position = pdb.gimp_image_get_item_position(image, drawable)
        group_new = pdb.gimp_layer_copy(drawable, TRUE)
        pdb.gimp_image_insert_layer(image, group_new, parent, position)
        group_new.name = "Group via Copy"

        pdb.gimp_selection_invert(image)
        sel_invert = pdb.gimp_selection_save(image)

        layers_list = find_layers_in_group(group_new, elements = [])
        for element in layers_list:

            pdb.gimp_image_select_item(image, CHANNEL_OP_REPLACE, sel_invert)

            pdb.gimp_layer_add_alpha(element)
            pdb.gimp_edit_named_cut(element, "buffer")
            pdb.gimp_buffer_delete("buffer")
            pdb.gimp_layer_resize_to_image_size(element)
            pdb.gimp_image_select_item(image, CHANNEL_OP_REPLACE, element)
            sel_size = pdb.gimp_selection_bounds(image)
            sel_width = sel_size[3] - sel_size[1]
            sel_height = sel_size[4] - sel_size[2]
            pdb.gimp_layer_resize(element, sel_width, sel_height, -sel_size[1], -sel_size[2])

        pdb.gimp_image_remove_channel(image, sel_invert)

        pdb.gimp_selection_none(image)
        pdb.gimp_image_set_active_layer(image, group_new)

        gimp.displays_flush()
        image.undo_group_end()
        gimp.context_pop()
        return

    parent = pdb.gimp_item_get_parent(drawable)
    position = pdb.gimp_image_get_item_position(image, drawable)
    layer_new = pdb.gimp_layer_copy(drawable, TRUE)
    pdb.gimp_image_insert_layer(image, layer_new, parent, position)
    layer_new.name = "Layer via Copy"

    pdb.gimp_selection_invert(image)
    pdb.gimp_layer_add_alpha(layer_new)
    pdb.gimp_edit_named_cut(layer_new, "buffer")
    pdb.gimp_buffer_delete("buffer")
    pdb.gimp_layer_resize_to_image_size(layer_new)
    pdb.gimp_image_select_item(image, CHANNEL_OP_REPLACE, layer_new)
    sel_size = pdb.gimp_selection_bounds(image)
    sel_width = sel_size[3] - sel_size[1]
    sel_height = sel_size[4] - sel_size[2]
    pdb.gimp_layer_resize(layer_new, sel_width, sel_height, -sel_size[1], -sel_size[2])

    pdb.gimp_selection_none(image)

    gimp.displays_flush()
    image.undo_group_end()
    gimp.context_pop()
    return

def python_fu_layer_via_cut(image, drawable):
    gimp.context_push()
    image.undo_group_start()
    pdb.gimp_message_get_handler(ERROR_CONSOLE)

    if not pdb.gimp_item_is_layer(drawable):
        show_message(0)
        image.undo_group_end()
        gimp.context_pop()
        return

    if pdb.gimp_selection_is_empty(image):
        show_message(1)
        image.undo_group_end()
        gimp.context_pop()
        return

    if pdb.gimp_item_is_group(drawable):

        parent = pdb.gimp_item_get_parent(drawable)
        position = pdb.gimp_image_get_item_position(image, drawable)
        group_new = pdb.gimp_layer_copy(drawable, TRUE)
        pdb.gimp_image_insert_layer(image, group_new, parent, position)
        group_new.name = "Group via Cut"

        sel = pdb.gimp_selection_save(image)
        pdb.gimp_selection_invert(image)
        sel_invert = pdb.gimp_selection_save(image)

        layers_list = find_layers_in_group(group_new, elements = [])
        for element in layers_list:

            pdb.gimp_image_select_item(image, CHANNEL_OP_REPLACE, sel_invert)

            pdb.gimp_layer_add_alpha(element)
            pdb.gimp_edit_named_cut(element, "buffer")
            pdb.gimp_buffer_delete("buffer")
            pdb.gimp_layer_resize_to_image_size(element)
            pdb.gimp_image_select_item(image, CHANNEL_OP_REPLACE, element)
            sel_size = pdb.gimp_selection_bounds(image)
            sel_width = sel_size[3] - sel_size[1]
            sel_height = sel_size[4] - sel_size[2]
            pdb.gimp_layer_resize(element, sel_width, sel_height, -sel_size[1], -sel_size[2])

        pdb.gimp_image_select_item(image, CHANNEL_OP_REPLACE, sel)
        layers_list = find_layers_in_group(drawable, elements = [])
        for element in layers_list:
            pdb.gimp_layer_add_alpha(element)
            pdb.gimp_edit_named_cut(element, "buffer")
            pdb.gimp_buffer_delete("buffer")

        pdb.gimp_image_remove_channel(image, sel)
        pdb.gimp_image_remove_channel(image, sel_invert)

        pdb.gimp_selection_none(image)
        pdb.gimp_image_set_active_layer(image, group_new)

        gimp.displays_flush()
        image.undo_group_end()
        gimp.context_pop()
        return

    parent = pdb.gimp_item_get_parent(drawable)
    position = pdb.gimp_image_get_item_position(image, drawable)
    layer_new = pdb.gimp_layer_copy(drawable, TRUE)
    pdb.gimp_image_insert_layer(image, layer_new, parent, position)
    layer_new.name = "Layer via Cut"

    sel = pdb.gimp_selection_save(image)
    pdb.gimp_selection_invert(image)
    sel_invert = pdb.gimp_selection_save(image)

    pdb.gimp_layer_add_alpha(layer_new)
    pdb.gimp_edit_named_cut(layer_new, "buffer")
    pdb.gimp_buffer_delete("buffer")
    pdb.gimp_layer_resize_to_image_size(layer_new)
    pdb.gimp_image_select_item(image, CHANNEL_OP_REPLACE, layer_new)
    sel_size = pdb.gimp_selection_bounds(image)
    sel_width = sel_size[3] - sel_size[1]
    sel_height = sel_size[4] - sel_size[2]
    pdb.gimp_layer_resize(layer_new, sel_width, sel_height, -sel_size[1], -sel_size[2])

    pdb.gimp_image_select_item(image, CHANNEL_OP_REPLACE, sel)
    pdb.gimp_layer_add_alpha(drawable)
    pdb.gimp_edit_named_cut(drawable, "buffer")
    pdb.gimp_buffer_delete("buffer")

    pdb.gimp_image_remove_channel(image, sel)
    pdb.gimp_image_remove_channel(image, sel_invert)

    pdb.gimp_selection_none(image)

    gimp.displays_flush()
    image.undo_group_end()
    gimp.context_pop()
    return

register (
    "python-fu-layer-via-copy",
    "Copy and move the selected area to a new layer in the same position.",
    "Copy and move the selected area to a new layer in the same position.",
    "Dmirty Dubyaga",
    "Dmitry Dubyaga <dmitry.dubyaga@gmail.com>",
    "04.06.2012",
    "Layer via Copy",
    "*",
    [
        (PF_IMAGE, "image", "", None),
        (PF_DRAWABLE, "drawable", "", None)
    ],
    [],
    python_fu_layer_via_copy, menu="<Image>/Layer/"
    )

register (
    "python-fu-layer-via-cut",
    "Cut and move the selected area to a new layer in the same position.",
    "Cut and move the selected area to a new layer in the same position.",
    "Dmirty Dubyaga",
    "Dmitry Dubyaga <dmitry.dubyaga@gmail.com>",
    "04.06.2012",
    "Layer via Cut",
    "*",
    [
        (PF_IMAGE, "image", "", None),
        (PF_DRAWABLE, "drawable", "", None)
    ],
    [],
    python_fu_layer_via_cut, menu="<Image>/Layer/"
    )

main()
